/*
 * SWTCheckboxPeer.java
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.EventQueue;
import java.awt.event.ItemEvent;
import java.awt.peer.CheckboxPeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTCheckboxPeer extends SWTComponentPeer implements CheckboxPeer {
  public SWTCheckboxPeer(SWTToolkit toolkit, Checkbox awtCheckBox) {
    super(toolkit, awtCheckBox);
    
    setLabel(awtCheckBox.getLabel());
    setState(awtCheckBox.getState());
  }

  @Override
  protected Widget createSWTWidget() {
    return new org.eclipse.swt.widgets.Button(getSWTParent(), 
        getAWTObject().getCheckboxGroup() != null ? SWT.RADIO : SWT.CHECK);
  }

  @Override
  public Checkbox getAWTObject() {
    return (Checkbox)super.getAWTObject();
  }
  
  @Override
  protected org.eclipse.swt.widgets.Button getSWTWidget() {
    return (org.eclipse.swt.widgets.Button)super.getSWTWidget();
  }
  
 @Override
  protected void handleSWTSelectionEvent(Event ev) {
    final boolean selected = getSWTWidget().getSelection();
    EventQueue.invokeLater(new Runnable() {
      public void run() {
        Checkbox awtCheckbox = getAWTObject();
        CheckboxGroup checkboxGroup = awtCheckbox.getCheckboxGroup();
        if ((checkboxGroup != null) 
            && (awtCheckbox == checkboxGroup.getSelectedCheckbox()) 
            && awtCheckbox.getState()) {
          return;
        }
        awtCheckbox.setState(selected);
        postAWTEvent(new ItemEvent(awtCheckbox, ItemEvent.ITEM_STATE_CHANGED, 
            getAWTObject().getLabel(), selected ? ItemEvent.SELECTED : ItemEvent.DESELECTED));
      }
    });
  }

  @Override
  public boolean isFocusable() {
    return true;
  }

  public void setCheckboxGroup(CheckboxGroup group) {
    // TODO Manage ourselves the group of radio buttons ?
  }

  public void setLabel(final String label) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setText(label);
        return null;
      }
    });
  }

  public void setState(final boolean state) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setSelection(state);
        return null;
      }
    });
  }
}
