/*
 * SWTChoicePeer.java 
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Choice;
import java.awt.EventQueue;
import java.awt.event.ItemEvent;
import java.awt.peer.ChoicePeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTChoicePeer extends SWTComponentPeer implements ChoicePeer {
  public SWTChoicePeer(SWTToolkit toolkit, final Choice awtChoice) {
    super(toolkit, awtChoice);
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        final Combo swtCombo = getSWTWidget();
        for (int i = 0; i < awtChoice.getItemCount(); i++) {
          swtCombo.add(awtChoice.getItem(i));
        }
        swtCombo.select(awtChoice.getSelectedIndex());
        return null;
      }
    });
  }

  @Override
  public Choice getAWTObject() {
    return (Choice)super.getAWTObject();
  }
  
  @Override
  protected Combo getSWTWidget() {
    return (Combo)super.getSWTWidget();
  }
  
  @Override
  protected Widget createSWTWidget() {
    return new Combo(getSWTParent(), SWT.READ_ONLY | SWT.DROP_DOWN);
  }

  @Override
  protected void handleSWTSelectionEvent(Event ev) {
    final Choice awtChoice = getAWTObject();
    final int selectedIndex = getSWTWidget().getSelectionIndex();
    // Update Choice selected index and post a ITEM_STATE_CHANGED AWT event 
    EventQueue.invokeLater(new Runnable() {
      public void run() {
        awtChoice.select(selectedIndex);
      }
    });
    postAWTEvent(new ItemEvent(awtChoice, ItemEvent.ITEM_STATE_CHANGED, 
        awtChoice.getItem(selectedIndex), ItemEvent.SELECTED));
  }

  @Override
  public boolean isFocusable() {
    return true;
  }

  @Override
  public boolean handlesWheelScrolling() {
   return true;
 }

  public void add(final String item, final int index) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().add(item, index);
        return null;
      }
    });
  }

  public void addItem(String item, int index) {
    add(item, index);
  }

  public void remove(final int index) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().remove(index);
        return null;
      }
    });
  }

  public void removeAll() {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().removeAll();
        return null;
      }
    });
  }

  public void select(final int index) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().select(index);
        return null;
      }
    });
  }
}
