/*
 * SWTColors.java
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;
import java.util.WeakHashMap;

import org.eclipse.swt.graphics.Device;

/**
 * @author Emmanuel Puybaret
 */
public class SWTColors {
  private static Map<Device, SWTColors> devicesColors = new HashMap<Device, SWTColors>();
  
  private final Device device; 
  private final Map<Color, SWTColor> swtColors;
  
  private SWTColors(Device device) {
    this.device = device;
    this.swtColors = new WeakHashMap<Color, SWTColor>();
  }
  
  public static SWTColors getInstance(Device device) {
    SWTColors deviceColors = devicesColors.get(device);
    if (deviceColors == null) {
      deviceColors = new SWTColors(device);
      devicesColors.put(device, deviceColors);
    }
    return deviceColors;
  }
  
  public org.eclipse.swt.graphics.Color getColor(Color awtColor) {
    SWTColor swtColor = swtColors.get(awtColor);
    if (swtColor == null) {
      swtColor = new SWTColor(new org.eclipse.swt.graphics.Color(this.device, 
          awtColor.getRed(), awtColor.getGreen(), awtColor.getBlue()));
      swtColors.put(awtColor, swtColor);
    }
    return swtColor.get();
  }
  
  /**
   * An SWT color wrapper that dispose the color when it's finalized.
   */
  private static class SWTColor {
    private final org.eclipse.swt.graphics.Color swtColor;
    
    public SWTColor(org.eclipse.swt.graphics.Color swtColor) {
      this.swtColor = swtColor;      
    }
    
    public org.eclipse.swt.graphics.Color get() {
      return this.swtColor;
    }

    @Override
    protected void finalize() throws Throwable {
      super.finalize();
      this.swtColor.dispose();
    }
  }
}
