/*
 * SWTDialogPeer.java 
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Dialog;
import java.awt.Insets;
import java.awt.peer.DialogPeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTDialogPeer extends SWTWindowPeer implements DialogPeer {
  public SWTDialogPeer(SWTToolkit toolkit, Dialog awtDialog) {
    super(toolkit, awtDialog);
    
    setTitle(awtDialog.getTitle());
  }

  @Override
  protected Widget createSWTWidget() {
    final Dialog awtDialog = getAWTObject();
    int style = SWT.DIALOG_TRIM;
    if (awtDialog.isResizable()) {
      style |= SWT.RESIZE;
    }
    if (awtDialog.isUndecorated()) {
      style = SWT.NONE;
    }
    if (awtDialog.isAlwaysOnTop()) {
      style |= SWT.ON_TOP;
    }
    if (awtDialog.isModal()) {
      style |= SWT.APPLICATION_MODAL;
    }
    Shell shell = new Shell((Shell)getSWTParent(), style);
    shell.setLayout(null);
    return shell;
  }
  
  @Override
  public Dialog getAWTObject() {
    return (Dialog)super.getAWTObject();
  }

  @Override
  public Insets getInsets() {
    return invokeInSWTDisplayThread(new Callable<Insets>() {
      public Insets call() {
        final Shell shell = getSWTWidget();
        org.eclipse.swt.graphics.Rectangle clientArea = shell.getClientArea();
        Point size = shell.getSize();
        return new Insets(clientArea.y, clientArea.x, 
            size.y - clientArea.height, size.x - clientArea.width);
      }
    });
  }

  public void setResizable(boolean resizeable) {
    // TODO No way to change RESIZE style ?
  }

  public void setTitle(final String title) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setText(title);
        return null;
      }
    });
  }
}
