/*
 * SWTFileDialogPeer.java 
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.AWTEvent;
import java.awt.BufferCapabilities;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.BufferCapabilities.FlipContents;
import java.awt.event.PaintEvent;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.ImageObserver;
import java.awt.image.ImageProducer;
import java.awt.image.VolatileImage;
import java.awt.peer.ContainerPeer;
import java.awt.peer.FileDialogPeer;
import java.io.File;
import java.io.FilenameFilter;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;

import sun.awt.CausedFocusEvent;

/**
 * @author Emmanuel Puybaret
 */
public class SWTFileDialogPeer implements FileDialogPeer {
  private SWTToolkit toolkit;
  private FileDialog awtFileDialog;

  public SWTFileDialogPeer(SWTToolkit toolkit, final FileDialog awtFileDialog) {
    // As SWT file dialog isn't a SWT widget, we can't make SWTFileDialogPeer inherit from SWTDialogPeer
    // Thus this class defines all FileDialogPeer methods and most of them are empty
    this.toolkit = toolkit;
    this.awtFileDialog = awtFileDialog;    
    setTitle(awtFileDialog.getTitle());
  }
  
  public Toolkit getToolkit() {
    return this.toolkit;
  }

  public void dispose() {
    // Nothing to do
  }

  public void setDirectory(String directory) {    
  }

  public void setFile(String file) {
  }

  public void setFilenameFilter(FilenameFilter filter) {
    // TODO Find a way to convert AWT File filters to SWT File filters
  }

  public void setTitle(String title) {
  }

  public void setVisible(final boolean visible) {
    final String title = this.awtFileDialog.getTitle();
    final String directory = this.awtFileDialog.getDirectory();
    final String file = this.awtFileDialog.getFile();
    
    String returnedValue = this.toolkit.invokeInSWTDisplayThread(new Callable<String>() {
      @SuppressWarnings("deprecation")
      public String call() {
        // TODO Find why sometimes it freezes application
        if (directory != null) {
          org.eclipse.swt.widgets.DirectoryDialog swtDirectoryDialog = new org.eclipse.swt.widgets.DirectoryDialog(
              ((SWTFramePeer)awtFileDialog.getParent().getPeer()).getSWTWidget(), SWT.NONE);          
          if (title != null) {
            swtDirectoryDialog.setText(title);
          }
          swtDirectoryDialog.setFilterPath(directory);
          return swtDirectoryDialog.open();
        } else {
          org.eclipse.swt.widgets.FileDialog swtFileDialog = new org.eclipse.swt.widgets.FileDialog(
                  ((SWTFramePeer)awtFileDialog.getParent().getPeer()).getSWTWidget(), 
                  awtFileDialog.getMode() == FileDialog.SAVE ? SWT.SAVE : SWT.OPEN);
          if (title != null) {
            swtFileDialog.setText(title);
          }
          if (file != null) {
            File filePath = new File(file);
            swtFileDialog.setFilterPath(filePath.getAbsolutePath());
            swtFileDialog.setFileName(filePath.getName());
          }
          return swtFileDialog.open();
        }
      }
    });
    if (directory != null) {
      awtFileDialog.setDirectory(returnedValue);
    } else {
      awtFileDialog.setFile(returnedValue);      
    }
  }

  public void hide() {
    setVisible(false);
  }
  
  public void show() {
    setVisible(true);
  }
  
  public void setBackground(final Color color) {
    // Not supported by SWT 
  }

  public void setForeground(final Color color) {
    // Not supported by SWT 
  }

  public void setFont(final Font font) {
    // Not supported by SWT 
  }

  final public void disable() {
    setEnabled(false);
  }

  final public void enable() {
    setEnabled(true);
  }

  public void setEnabled(final boolean enabled) {
    // Not supported by SWT 
  }

  public Rectangle getBounds() {
    // Not supported by SWT 
    return new Rectangle();
  }


  public Dimension preferredSize() {
    return getPreferredSize();
  }

  public Dimension getPreferredSize() {
    // Not supported by SWT 
    return new Dimension(0, 0);
  }

  public Dimension minimumSize() {
    return getMinimumSize();
  }

  public Dimension getMinimumSize() {
    return getPreferredSize();
  }

  public void reshape(final int x, final int y, final int width, final int height) {
    // Not supported by SWT 
  }

  public void setBounds(int x, int y, int width, int height, int op) {
    reshape(x, y, width, height);
  }

  public boolean canDetermineObscurity() {
    return false;
  }

  public boolean isObscured() {
    return false;
  }

  public Image createImage(ImageProducer producer) {
    return this.toolkit.createImage(producer);
  }

  public Image createImage(int width, int height) {
    return new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
  }

  public int checkImage(Image image, int width, int height, ImageObserver observer) {
    return getToolkit().checkImage(image, width, height, observer);
  }

  public boolean prepareImage(Image image, int width, int height, ImageObserver observer) {
    return getToolkit().prepareImage(image, width, height, observer);
  }

  public VolatileImage createVolatileImage(int width, int height) {
    // TODO Manage volatile images
    return null;
  }

  public void flip(FlipContents flipAction) {
    // TODO Manage volatile images
  }

  public Image getBackBuffer() {
    // TODO Manage volatile images
    return null;
  }

  public void createBuffers(int numBuffers, BufferCapabilities caps) {
    // TODO Manage volatile images
  }

  public void destroyBuffers() {
    // TODO Manage volatile images
  }

  public ColorModel getColorModel() {
    return getGraphicsConfiguration().getColorModel();
  }

  public FontMetrics getFontMetrics(Font font) {
    return this.toolkit.getFontMetrics(font);
  }

  public GraphicsConfiguration getGraphicsConfiguration() {
    return GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
  }

  public Graphics getGraphics() {
    // Not supported by SWT 
    return null;
  }

  public void print(Graphics g) {
  }

  public void paint(Graphics g) {
  }

  public void repaint(long tm, int x, int y, int width, int height) {
  }

  public boolean isPaintPending() {
    return false;
  }

  public void coalescePaintEvent(PaintEvent e) {
  }

  public void handleEvent(AWTEvent ev) {
  }

  public void beginLayout() {
  }

  public void endLayout() {
  }
  
  public void layout() {
  }

  public boolean isReparentSupported() {
    return false;
  }

  public void reparent(ContainerPeer newContainer) {
    // Not supported by SWT 
  }

  public boolean handlesWheelScrolling() {
    return false;
  }

  public boolean isFocusable() {
    return true;
  }

  public boolean requestFocus(Component lightweightChild,
                              boolean temporary,
                              boolean focusedWindowChangeAllowed,
                              long time) {
    return false;
  }

  public boolean requestFocus(Component lightweightChild,
                              boolean temporary,
                              boolean focusedWindowChangeAllowed,
                              long time, CausedFocusEvent.Cause cause) {
    return false;
  }
  
  public void updateCursorImmediately() {
    // Unsupported by SWT
  }

  public Point getLocationOnScreen() {
    return new Point(0, 0);
  }

  public boolean isRestackSupported() {
    return false;
  }

  public void restack() {
  }

  public void setResizable(boolean resizeable) {
    // Not supported by SWT 
  }

  public boolean requestWindowFocus() {
    return false;
  }

  public void setAlwaysOnTop(boolean alwaysOnTop) {
  }

  public void setModalBlocked(Dialog blocker, boolean blocked) {
  }

  public void toBack() {
  }

  public void toFront() {
  }

  public void updateFocusableWindowState() {
  }

  public void updateIconImages() {
  }

  public void updateMinimumSize() {
  }

  public void beginValidate() {
  }

  public void endValidate() {
  }

  public Insets getInsets() {
    return new Insets(0, 0, 0, 0);
  }

  public Insets insets() {
    return getInsets();
  }
}
