/*
 * SWTFonts.java
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Font;
import java.util.HashMap;
import java.util.Map;
import java.util.WeakHashMap;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Device;

/**
 * @author Emmanuel Puybaret
 */
public class SWTFonts {
  private static Map<Device, SWTFonts> devicesFonts = new HashMap<Device, SWTFonts>();
  
  private final Device device; 
  private final Map<Font, SWTFont> swtFonts;
  
  private SWTFonts(Device device) {
    this.device = device;
    this.swtFonts = new WeakHashMap<Font, SWTFont>();
  }
  
  public static SWTFonts getInstance(Device device) {
    SWTFonts deviceFonts = devicesFonts.get(device);
    if (deviceFonts == null) {
      deviceFonts = new SWTFonts(device);
      devicesFonts.put(device, deviceFonts);
    }
    return deviceFonts;
  }
  
  public org.eclipse.swt.graphics.Font getFont(Font awtFont) {
    SWTFont swtFont = swtFonts.get(awtFont);
    if (swtFont == null) {
      int swtStyle = 0;
      if (awtFont.getStyle() == Font.PLAIN) {
        swtStyle = SWT.NORMAL;
      } else {
        if ((awtFont.getStyle() & Font.ITALIC) == Font.ITALIC) {
          swtStyle = SWT.ITALIC;
        }
        if ((awtFont.getStyle() & Font.BOLD) == Font.BOLD) {
          swtStyle |= SWT.BOLD;
        }
      }
      swtFont = new SWTFont(new org.eclipse.swt.graphics.Font(this.device, 
          awtFont.getName(), awtFont.getSize(), swtStyle));
      swtFonts.put(awtFont, swtFont);
    }
    return swtFont.get();
  }
  
  /**
   * An SWT color wrapper that dispose the color when it's finalized.
   */
  private static class SWTFont {
    private final org.eclipse.swt.graphics.Font swtFont;
    
    public SWTFont(org.eclipse.swt.graphics.Font swtFont) {
      this.swtFont = swtFont;      
    }
    
    public org.eclipse.swt.graphics.Font get() {
      return this.swtFont;
    }

    @Override
    protected void finalize() throws Throwable {
      super.finalize();
      this.swtFont.dispose();
    }
  }
}
