/*
 * SWTFramePeer.java
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Color;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Image;
import java.awt.Insets;
import java.awt.MenuBar;
import java.awt.Rectangle;
import java.awt.peer.FramePeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTFramePeer extends SWTWindowPeer implements FramePeer {
  public SWTFramePeer(SWTToolkit toolkit, Frame awtFrame) {
    super(toolkit, awtFrame);
    
    setTitle(awtFrame.getTitle());
    setState(awtFrame.getState());
    setIconImage(awtFrame.getIconImage());
  }

  @Override
  protected Widget createSWTWidget() {
    final Frame awtFrame = getAWTObject();
    int style = SWT.SHELL_TRIM;
    if (!awtFrame.isResizable()) {
      style &= ~SWT.RESIZE;
    }
    if (awtFrame.isUndecorated()) {
      style = SWT.NONE;
    }
    if (awtFrame.isAlwaysOnTop()) {
      style |= SWT.ON_TOP;
    }
    Shell shell = new Shell(((SWTToolkit)getToolkit()).getSWTDisplay(), style);
    shell.setLayout(null);
    
    // Set default colors and font if they are not set     
    Color background = awtFrame.getBackground();
    if (background == null) {
      org.eclipse.swt.graphics.Color swtColor = getToolkit().getSWTDisplay().getSystemColor(SWT.COLOR_WIDGET_BACKGROUND);
      final Color awtColor = new Color(swtColor.getRed(), swtColor.getGreen(), swtColor.getBlue());
      EventQueue.invokeLater(new Runnable() {
        public void run() {
          awtFrame.setBackground(awtColor);
        }
      });
    }
    Color foreground = awtFrame.getForeground();
    if (foreground == null) {
      org.eclipse.swt.graphics.Color swtColor = getToolkit().getSWTDisplay().getSystemColor(SWT.COLOR_WIDGET_FOREGROUND);
      final Color awtColor = new Color(swtColor.getRed(), swtColor.getGreen(), swtColor.getBlue());
      EventQueue.invokeLater(new Runnable() {
        public void run() {
          awtFrame.setForeground(awtColor);
        }
      });
    }
    Font font = awtFrame.getFont();
    if (font == null) {
      final org.eclipse.swt.graphics.FontData swtFontData = getToolkit().getSWTDisplay().getSystemFont().getFontData() [0];
      final String fontName = swtFontData.getName();
      int awtStyle = 0;
      if (swtFontData.getStyle() == SWT.NORMAL) {
        awtStyle = Font.PLAIN;
      } else {
        if ((swtFontData.getStyle() & SWT.ITALIC) == SWT.ITALIC) {
          awtStyle = Font.ITALIC;
        }
        if ((swtFontData.getStyle() & SWT.BOLD) == SWT.BOLD) {
          awtStyle |= Font.BOLD;
        }
      }
      final int fontStyle = awtStyle;
      final int fontSize = swtFontData.getHeight();
      EventQueue.invokeLater(new Runnable() {
        public void run() {
          Font awtFont = new Font(fontName, fontStyle, fontSize);
          awtFrame.setFont(awtFont);
        }
      });
    }
    return shell;
  }
  
  @Override
  public Frame getAWTObject() {
    return (Frame)super.getAWTObject();
  }

  public void setTitle(final String title) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setText(title);
        return null;
      }
    });
  }

  public int getState() {
    return invokeInSWTDisplayThread(new Callable<Integer>() {
      public Integer call() {
        Shell shell = getSWTWidget();
        if (shell.getMaximized()) {
          return Frame.MAXIMIZED_BOTH;
        } else if (shell.getMinimized()) {
          return Frame.ICONIFIED;
        }
        return Frame.NORMAL;
      }
    });
  }

  public void setState(final int state) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        Shell shell = getSWTWidget();
        if ((state & Frame.MAXIMIZED_HORIZ) == Frame.MAXIMIZED_HORIZ
            || (state & Frame.MAXIMIZED_VERT) == Frame.MAXIMIZED_VERT) {
          shell.setMaximized(true);
        } else if ((state & Frame.ICONIFIED) == Frame.ICONIFIED) {
          shell.setMinimized(true);
        } else {
          if (shell.getMaximized()) {
            shell.setMaximized(false);
          } else if (shell.getMinimized()) {
            shell.setMinimized(false);
          }
        }
        return null;  
      }
    });
  }
  
  @Override
  public Insets getInsets() {
    return invokeInSWTDisplayThread(new Callable<Insets>() {
      public Insets call() {
        final Shell shell = getSWTWidget();
        org.eclipse.swt.graphics.Rectangle clientArea = shell.getClientArea();
        Point size = shell.getSize();
        return new Insets(clientArea.y, clientArea.x, 
            size.y - clientArea.height, size.x - clientArea.width);
      }
    });
  }

  @SuppressWarnings("deprecation")
  public void setMenuBar(final MenuBar menuBar) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        SWTMenuBarPeer menuBarPeer = (SWTMenuBarPeer)menuBar.getPeer();
        menuBarPeer.setSWTParent(getSWTWidget());
        menuBarPeer.initSWTWidget();
        return null;
      }
    });
  }

  public void setBoundsPrivate(final int x, final int y, final int width, final int height) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setBounds(x, y, width, height);
        return null;
      }
    });
  }

  public Rectangle getBoundsPrivate() {
    return invokeInSWTDisplayThread(new Callable<Rectangle>() {
      public Rectangle call() {
        org.eclipse.swt.graphics.Rectangle bounds = getSWTWidget().getBounds();
        return new Rectangle(bounds.x, bounds.y, bounds.width, bounds.height);
      }
    });
  }
  
  public void setIconImage(Image awtImage) {
    final org.eclipse.swt.graphics.Image swtImage = getSWTImage(awtImage);
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        org.eclipse.swt.graphics.Image oldImage = getSWTWidget().getImage();
        getSWTWidget().setImage(swtImage);
        if (oldImage != null) {
          oldImage.dispose();
        }
        return null;
      }
    });
  }

  public void setMaximizedBounds(Rectangle bounds) {
    // TODO Not sure we just need to change frame bounds
    // setBoundsPrivate(bounds.x, bounds.y, bounds.width, bounds.height);
  }
  
  public void setResizable(boolean resizeable) {
    // TODO No way to change RESIZE style ?
  }

  public void updateAlwaysOnTop() {
    // TODO No way to change ON_TOP style ?
  }
}
