/*
 * SWTListPeer.java
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.List;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.peer.ListPeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTListPeer extends SWTComponentPeer implements ListPeer {

  public SWTListPeer(SWTToolkit toolkit, final List awtList) {
    super(toolkit, awtList);
    
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setItems(awtList.getItems());
        getSWTWidget().setSelection(awtList.getSelectedIndexes());
        return null;
      }
    });
  }

  @Override
  protected Widget createSWTWidget() {
    return new org.eclipse.swt.widgets.List(getSWTParent(), 
        (getAWTObject().isMultipleMode() ? SWT.MULTI : SWT.SINGLE) | SWT.BORDER | SWT.V_SCROLL);
  }

  @Override
  public List getAWTObject() {
    return (List)super.getAWTObject();
  }
  
  @Override
  protected org.eclipse.swt.widgets.List getSWTWidget() {
    return (org.eclipse.swt.widgets.List)super.getSWTWidget();
  }
  
  @Override
  protected void handleSWTDefaultSelectionEvent(Event ev) {
    // TODO Search the last selected item ? 
    final int index = ((org.eclipse.swt.widgets.List)ev.widget).getSelectionIndex();
    postAWTEvent(new ActionEvent(getAWTObject(), ActionEvent.ACTION_PERFORMED, 
        getAWTObject().getItem(index), ev.time, getModifiers(ev.stateMask)));
  }

  @Override
  protected void handleSWTSelectionEvent(Event ev) {
    // TODO Should track selection changes to be able to post DESELECTED events when
    // multiple items are selected
    final int index = getSWTWidget().getSelectionIndex();
    final boolean selected = getSWTWidget().isSelected(index);
    postAWTEvent(new ItemEvent(getAWTObject(), ItemEvent.ITEM_STATE_CHANGED, 
        index, selected ? ItemEvent.SELECTED : ItemEvent.DESELECTED));
  }

  @Override
  public boolean isFocusable() {
    return true;
  }

  @Override
   public boolean handlesWheelScrolling() {
    return true;
  }

  public void add(final String item, final int index) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().add(item, index);
        return null;
      }
    });
  }

  public void addItem(String item, int index) {
    add(item, index);
  }

  public void delItems(final int start, final int end) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().remove(start, end);
        return null;
      }
    });
  }

  public Dimension getMinimumSize(int rows) {
    return getPreferredSize(rows);
  }

  public Dimension minimumSize(int v) {
    return getMinimumSize(v);
  }
  
  public Dimension getPreferredSize(int rows) {
    FontMetrics fontMetrics = getFontMetrics(getAWTObject().getFont());
    return new Dimension(20 + fontMetrics.stringWidth("0123456789"),
                         ((fontMetrics.getHeight() + 3) * rows) + 6);
  }

  public Dimension preferredSize(int v) {
    return getPreferredSize(v);
  }
  
  public void deselect(final int index) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().deselect(index);
        return null;
      }
    });
  }

  public void select(final int index) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().select(index);
        return null;
      }
    });
  }

  public int [] getSelectedIndexes() {
    return invokeInSWTDisplayThread(new Callable<int []>() {
      public int [] call() {
        return getSWTWidget().getSelectionIndices();
      }
    });
  }

  public void makeVisible(final int index) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setTopIndex(index);
        return null;
      }
    });
  }

  public void removeAll() {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().removeAll();
        return null;
      }
    });
  }

  public void clear() {
    removeAll();
  }

  public void setMultipleMode(boolean b) {
    // TODO No way to change MULTI style ?
  }

  public void setMultipleSelections(boolean v) {
    setMultipleMode(v);
  }
}
