/*
 * SWTMenuComponentPeer.java
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Font;
import java.awt.MenuComponent;
import java.awt.peer.MenuComponentPeer;

import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public abstract class SWTMenuComponentPeer extends SWTObjectPeer implements MenuComponentPeer {
  private Widget swtParent;

  public SWTMenuComponentPeer(SWTToolkit toolkit, final MenuComponent awtComponent) {
    super(toolkit, awtComponent);
    Font font = awtComponent.getFont();
    if (font != null) {
      setFont(font);
    }
  }

  public void setFont(Font font) {
    // Unsupported font change on SWT menus
  }
  
  /**
   * Sets the SWT parent composite of this peer SWT widget.
   * AWT menu bars, menus, menu items... peers are created in a different 
   * way by AWT than other frame components, meaning that all menu objects 
   * peers are created first before their menu bar gets attached to 
   * a <code>Frame</code> instance. As SWT requires a parent composite 
   * to create menu widgets, we can create the widgets only once
   * we know at which frame they will be attached.
   */
  void setSWTParent(Widget menuParent) {
    this.swtParent = menuParent;
  }

  /**
   * Returns the SWT parent composite of this peer SWT widget or <code>null</code>
   * if it doesn't exist yet.
   */
  @Override
  protected Widget getSWTParent() {
    return this.swtParent;
  }
}
