/*
 * SWTMenuItemPeer.java 
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.peer.MenuItemPeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTMenuItemPeer extends SWTMenuComponentPeer implements MenuItemPeer {
  public SWTMenuItemPeer(SWTToolkit toolkit, MenuItem awtMenuItem) {
    super(toolkit, awtMenuItem);
  }

  @Override
  protected Widget createSWTWidget() {
    MenuItem awtMenuItem = getAWTObject();
    org.eclipse.swt.widgets.Menu parent = (org.eclipse.swt.widgets.Menu)getSWTParent();
    org.eclipse.swt.widgets.MenuItem swtMenuItem;
    if (awtMenuItem.getLabel().equals("-")) {
      swtMenuItem = new org.eclipse.swt.widgets.MenuItem(parent, SWT.SEPARATOR);
    } else {
      swtMenuItem = new org.eclipse.swt.widgets.MenuItem(parent, SWT.PUSH);
      setLabel(swtMenuItem, awtMenuItem.getLabel(), awtMenuItem.getShortcut());
      setEnabled(swtMenuItem, awtMenuItem.isEnabled());
    }
    return swtMenuItem;
  }

  @Override
  protected MenuItem getAWTObject() {
    return (MenuItem)super.getAWTObject();
  }
  
  @Override
  protected void handleSWTSelectionEvent(Event ev) {
    postAWTEvent(new ActionEvent(getAWTObject(), ActionEvent.ACTION_PERFORMED, 
        getAWTObject().getActionCommand(), ev.time, getModifiers(ev.stateMask)));
  }
  
  public void setEnabled(final boolean enabled) {
    org.eclipse.swt.widgets.MenuItem swtMenuItem = (org.eclipse.swt.widgets.MenuItem)getSWTWidget();
    if (swtMenuItem != null) {
      setEnabled((org.eclipse.swt.widgets.MenuItem)getSWTWidget(), enabled);
    }
  }

  protected void setEnabled(final org.eclipse.swt.widgets.MenuItem swtMenuItem, final boolean enabled) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        swtMenuItem.setEnabled(enabled);
        return null;
      }
    });
  }
  
  public void disable() {
    setEnabled(false);
  }

  public void enable() {
    setEnabled(true);
  }

  public void setLabel(final String label) {
    org.eclipse.swt.widgets.MenuItem swtMenuItem = (org.eclipse.swt.widgets.MenuItem)getSWTWidget();
    if (swtMenuItem != null) {
      setLabel(swtMenuItem, label, getAWTObject().getShortcut());
    }
  }

  protected void setLabel(final org.eclipse.swt.widgets.MenuItem swtMenuItem, 
                          final String label, final MenuShortcut shortcut) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        swtMenuItem.setText(label);
        if (shortcut != null) {
          int menuShortcutKeyMask = getToolkit().getMenuShortcutKeyMask();
          if (menuShortcutKeyMask == KeyEvent.META_MASK) {
            menuShortcutKeyMask = SWT.COMMAND;
          } else {
            menuShortcutKeyMask = SWT.CTRL;
          }
          if (shortcut.usesShiftModifier()) {
            swtMenuItem.setAccelerator(getSWTKeyCode(shortcut.getKey()) | SWT.SHIFT | menuShortcutKeyMask);
          } else {
            swtMenuItem.setAccelerator(getSWTKeyCode(shortcut.getKey()) | menuShortcutKeyMask);
          }
        }
        return null;
      }
    });
  }
}
