/*
 * SWTMenuPeer.java 
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.peer.MenuPeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTMenuPeer extends SWTMenuItemPeer implements MenuPeer {
  private org.eclipse.swt.widgets.MenuItem swtMenuItem;

  public SWTMenuPeer(SWTToolkit toolkit, Menu awtMenu) {
    super(toolkit, awtMenu);
  }

  @Override
  protected Widget createSWTWidget() {
    this.swtMenuItem = new org.eclipse.swt.widgets.MenuItem((org.eclipse.swt.widgets.Menu)getSWTParent(), SWT.CASCADE);
    Menu awtMenu = (Menu)getAWTObject();
    setLabel(this.swtMenuItem, awtMenu.getLabel(), null);
    setEnabled(this.swtMenuItem, awtMenu.isEnabled());
    org.eclipse.swt.widgets.Menu swtMenu = new org.eclipse.swt.widgets.Menu(this.swtMenuItem);
    this.swtMenuItem.setMenu(swtMenu);
    for (int i = 0; i < awtMenu.getItemCount(); i++) {
      addItem(awtMenu.getItem(i), swtMenu);
    }
    return swtMenu;
  }
  
  /**
   * Disposes the SWT widget associated with this peer. 
   */
  public void dispose() {
    super.dispose();
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        if (swtMenuItem != null) {
          swtMenuItem.dispose();
          swtMenuItem = null;
        }
        return null;
      }
    });
  }

  public void addItem(MenuItem awtMenuItem) {
    addItem(awtMenuItem, (org.eclipse.swt.widgets.Menu)getSWTWidget());
  }

  public void addItem(final MenuItem awtMenuItem, final org.eclipse.swt.widgets.Menu swtMenu) {
    if (swtMenu != null) {
      invokeInSWTDisplayThread(new Callable<Void>() {
        @SuppressWarnings("deprecation")
        public Void call() {
          SWTMenuItemPeer menuPeer = (SWTMenuItemPeer)awtMenuItem.getPeer();
          menuPeer.setSWTParent(swtMenu);
          menuPeer.initSWTWidget();          
          return null;
        }
      });
    }
  }

  public void addSeparator() {
    // Never called in AWT !
  }

  public void delItem(final int index) {
    // Nothing to do, item will be automatically deleted once disposed
  }
}
