/*
 * SWTComponentPeer.java
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.AWTEvent;
import java.awt.Image;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.image.ImageObserver;
import java.awt.image.PixelGrabber;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public abstract class SWTObjectPeer {
  private static final int [][] SWT_AWT_KEY_CODES = {
    {SWT.CR, KeyEvent.VK_ENTER},
    {SWT.BS, KeyEvent.VK_BACK_SPACE},
    {SWT.TAB, KeyEvent.VK_TAB},
    {SWT.ESC, KeyEvent.VK_ESCAPE},
    {SWT.CTRL, KeyEvent.VK_CONTROL},
    {SWT.ALT, KeyEvent.VK_ALT},
    {SWT.COMMAND, KeyEvent.VK_META},
    {SWT.SHIFT, KeyEvent.VK_SHIFT},
    {SWT.CAPS_LOCK, KeyEvent.VK_CAPS_LOCK},
    {SWT.NUM_LOCK, KeyEvent.VK_NUM_LOCK},
    {SWT.SCROLL_LOCK, KeyEvent.VK_SCROLL_LOCK},
    {SWT.PRINT_SCREEN, KeyEvent.VK_PRINTSCREEN},
    {SWT.PAUSE, KeyEvent.VK_PAUSE},
    {SWT.CANCEL, KeyEvent.VK_CANCEL},
    {SWT.SEPARATOR, KeyEvent.VK_SEPARATOR},
    {SWT.HELP, KeyEvent.VK_HELP},
    {SWT.PAGE_UP, KeyEvent.VK_PAGE_UP},
    {SWT.PAGE_DOWN, KeyEvent.VK_PAGE_DOWN},
    {SWT.END, KeyEvent.VK_END},
    {SWT.HOME, KeyEvent.VK_HOME},
    {SWT.INSERT, KeyEvent.VK_INSERT},
    {SWT.DEL, KeyEvent.VK_DELETE},
    {SWT.ARROW_LEFT, KeyEvent.VK_LEFT},
    {SWT.ARROW_UP, KeyEvent.VK_UP},
    {SWT.ARROW_RIGHT, KeyEvent.VK_RIGHT},
    {SWT.ARROW_DOWN, KeyEvent.VK_DOWN},
    {SWT.KEYPAD_0, KeyEvent.VK_NUMPAD0},
    {SWT.KEYPAD_1, KeyEvent.VK_NUMPAD1},
    {SWT.KEYPAD_2, KeyEvent.VK_NUMPAD2},
    {SWT.KEYPAD_3, KeyEvent.VK_NUMPAD3},
    {SWT.KEYPAD_4, KeyEvent.VK_NUMPAD4},
    {SWT.KEYPAD_5, KeyEvent.VK_NUMPAD5},
    {SWT.KEYPAD_6, KeyEvent.VK_NUMPAD6},
    {SWT.KEYPAD_7, KeyEvent.VK_NUMPAD7},
    {SWT.KEYPAD_8, KeyEvent.VK_NUMPAD8},
    {SWT.KEYPAD_9, KeyEvent.VK_NUMPAD9},
    {SWT.KEYPAD_DECIMAL, KeyEvent.VK_DECIMAL},
    {SWT.KEYPAD_MULTIPLY, KeyEvent.VK_MULTIPLY},
    {SWT.KEYPAD_ADD, KeyEvent.VK_ADD},
    {SWT.KEYPAD_DIVIDE, KeyEvent.VK_DIVIDE},
    {SWT.KEYPAD_SUBTRACT, KeyEvent.VK_SUBTRACT},
    {SWT.KEYPAD_EQUAL, KeyEvent.VK_EQUALS},
    {SWT.KEYPAD_CR, KeyEvent.VK_ENTER},
    {SWT.F1, KeyEvent.VK_F1},
    {SWT.F2, KeyEvent.VK_F2},
    {SWT.F3, KeyEvent.VK_F3},
    {SWT.F4, KeyEvent.VK_F4},
    {SWT.F5, KeyEvent.VK_F5},
    {SWT.F6, KeyEvent.VK_F6},
    {SWT.F7, KeyEvent.VK_F7},
    {SWT.F8, KeyEvent.VK_F8},
    {SWT.F9, KeyEvent.VK_F9},
    {SWT.F10, KeyEvent.VK_F10},
    {SWT.F11, KeyEvent.VK_F11},
    {SWT.F12, KeyEvent.VK_F12},
    {SWT.F13, KeyEvent.VK_F13},
    {SWT.F14, KeyEvent.VK_F14},
    {SWT.F15, KeyEvent.VK_F15},
    {'0', KeyEvent.VK_0},
    {'1', KeyEvent.VK_1},
    {'2', KeyEvent.VK_2},
    {'3', KeyEvent.VK_3},
    {'4', KeyEvent.VK_4},
    {'5', KeyEvent.VK_5},
    {'6', KeyEvent.VK_6},
    {'7', KeyEvent.VK_7},
    {'8', KeyEvent.VK_8},
    {'9', KeyEvent.VK_9},
    {'A', KeyEvent.VK_A},
    {'B', KeyEvent.VK_B},
    {'C', KeyEvent.VK_C},
    {'D', KeyEvent.VK_D},
    {'E', KeyEvent.VK_E},
    {'F', KeyEvent.VK_F},
    {'G', KeyEvent.VK_G},
    {'H', KeyEvent.VK_H},
    {'I', KeyEvent.VK_I},
    {'J', KeyEvent.VK_J},
    {'K', KeyEvent.VK_K},
    {'L', KeyEvent.VK_L},
    {'M', KeyEvent.VK_M},
    {'N', KeyEvent.VK_N},
    {'O', KeyEvent.VK_O},
    {'P', KeyEvent.VK_P},
    {'Q', KeyEvent.VK_Q},
    {'R', KeyEvent.VK_R},
    {'S', KeyEvent.VK_S},
    {'T', KeyEvent.VK_T},
    {'U', KeyEvent.VK_U},
    {'V', KeyEvent.VK_V},
    {'W', KeyEvent.VK_W},
    {'X', KeyEvent.VK_X},
    {'Y', KeyEvent.VK_Y},
    {'Z', KeyEvent.VK_Z},
    {' ', KeyEvent.VK_SPACE},
    {',', KeyEvent.VK_COMMA},
    {';', KeyEvent.VK_SEMICOLON},
    {'.', KeyEvent.VK_PERIOD},
    {':', KeyEvent.VK_COLON},
    {'!', KeyEvent.VK_EXCLAMATION_MARK},
    {'_', KeyEvent.VK_UNDERSCORE},
    {'`', KeyEvent.VK_BACK_QUOTE},
    {'\'', KeyEvent.VK_QUOTE},
    {'"', KeyEvent.VK_QUOTEDBL},
    {'=', KeyEvent.VK_EQUALS},
    {'+', KeyEvent.VK_PLUS},
    {'-', KeyEvent.VK_MINUS},
    {'*', KeyEvent.VK_ASTERISK},
    {'/', KeyEvent.VK_SLASH},
    {'\\', KeyEvent.VK_BACK_SLASH},
    {'(', KeyEvent.VK_LEFT_PARENTHESIS},
    {')', KeyEvent.VK_RIGHT_PARENTHESIS},
    {'[', KeyEvent.VK_OPEN_BRACKET},
    {']', KeyEvent.VK_CLOSE_BRACKET},
    {'<', KeyEvent.VK_LESS},
    {'>', KeyEvent.VK_GREATER},
    {'{', KeyEvent.VK_BRACELEFT},
    {'}', KeyEvent.VK_BRACERIGHT},
    {'@', KeyEvent.VK_AT},
    {'^', KeyEvent.VK_CIRCUMFLEX},
    {'$', KeyEvent.VK_DOLLAR},
    {'\u20ac', KeyEvent.VK_EURO_SIGN},
    {'#', KeyEvent.VK_NUMBER_SIGN},
    {'&', KeyEvent.VK_AMPERSAND},
// Unknown SWT key codes
//  {SWT., KeyEvent.VK_F16},
//  {SWT., KeyEvent.VK_F17},
//  {SWT., KeyEvent.VK_F18},
//  {SWT., KeyEvent.VK_F19},
//  {SWT., KeyEvent.VK_F20},
//  {SWT., KeyEvent.VK_F21},
//  {SWT., KeyEvent.VK_F22},
//  {SWT., KeyEvent.VK_F23},
//  {SWT., KeyEvent.VK_F24},
//  {SWT., KeyEvent.VK_CLEAR},
//  {'', KeyEvent.VK_INVERTED_EXCLAMATION_MARK},
//  {'', KeyEvent.VK_KP_UP},
//  {'', KeyEvent.VK_KP_DOWN},
//  {'', KeyEvent.VK_KP_LEFT},
//  {'', KeyEvent.VK_KP_RIGHT},
//  {'', KeyEvent.VK_DEAD_GRAVE},
//  {'', KeyEvent.VK_DEAD_ACUTE},
//  {'', KeyEvent.VK_DEAD_CIRCUMFLEX},
//  {'', KeyEvent.VK_DEAD_TILDE},
//  {'', KeyEvent.VK_DEAD_MACRON},
//  {'', KeyEvent.VK_DEAD_BREVE},
//  {'', KeyEvent.VK_DEAD_ABOVEDOT},
//  {'', KeyEvent.VK_DEAD_DIAERESIS},
//  {'', KeyEvent.VK_DEAD_ABOVERING},
//  {'', KeyEvent.VK_DEAD_DOUBLEACUTE},
//  {'', KeyEvent.VK_DEAD_CARON},
//  {'', KeyEvent.VK_DEAD_CEDILLA},
//  {'', KeyEvent.VK_DEAD_OGONEK},
//  {'', KeyEvent.VK_DEAD_IOTA},
//  {'', KeyEvent.VK_DEAD_VOICED_SOUND},
//  {'', KeyEvent.VK_DEAD_SEMIVOICED_SOUND},
//  {'', KeyEvent.VK_WINDOWS},
//  {'', KeyEvent.VK_CONTEXT_MENU},
//  {'', KeyEvent.VK_FINAL},
//  {'', KeyEvent.VK_CONVERT},
//  {'', KeyEvent.VK_NONCONVERT},
//  {'', KeyEvent.VK_ACCEPT},
//  {'', KeyEvent.VK_MODECHANGE},
//  {'', KeyEvent.VK_KANA},
//  {'', KeyEvent.VK_KANJI},
//  {'', KeyEvent.VK_ALPHANUMERIC},
//  {'', KeyEvent.VK_KATAKANA},
//  {'', KeyEvent.VK_HIRAGANA},
//  {'', KeyEvent.VK_FULL_WIDTH},
//  {'', KeyEvent.VK_HALF_WIDTH},
//  {'', KeyEvent.VK_ROMAN_CHARACTERS},
//  {'', KeyEvent.VK_ALL_CANDIDATES},
//  {'', KeyEvent.VK_PREVIOUS_CANDIDATE},
//  {'', KeyEvent.VK_CODE_INPUT},
//  {'', KeyEvent.VK_JAPANESE_KATAKANA},
//  {'', KeyEvent.VK_JAPANESE_HIRAGANA},
//  {'', KeyEvent.VK_JAPANESE_ROMAN},
//  {'', KeyEvent.VK_KANA_LOCK},
//  {'', KeyEvent.VK_INPUT_METHOD_ON_OFF},
//  {'', KeyEvent.VK_CUT},
//  {'', KeyEvent.VK_COPY},
//  {'', KeyEvent.VK_PASTE},
//  {'', KeyEvent.VK_UNDO},
//  {'', KeyEvent.VK_AGAIN},
//  {'', KeyEvent.VK_FIND},
//  {'', KeyEvent.VK_PROPS},
//  {'', KeyEvent.VK_STOP},
//  {'', KeyEvent.VK_COMPOSE},
//  {'', KeyEvent.VK_ALT_GRAPH},
//  {'', KeyEvent.VK_BEGIN},
  };
  
  private SWTToolkit toolkit;
  private Object     awtObject;
  private Widget     swtWidget;

  public SWTObjectPeer(final SWTToolkit toolkit, final Object awtObject) {
    this.toolkit = toolkit;
    this.awtObject = awtObject;
  }

  /**
   * Calls {@link #createSWTWidget() createSWTWidget} method and registers
   * the created SWT widget in toolkit. This method should be called
   * only once the SWT parent of the created widget is known. 
   */
  protected void initSWTWidget() {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        swtWidget = createSWTWidget();        
        if (swtWidget != null) {
          toolkit.registerSWTWidget(swtWidget, SWTObjectPeer.this);
        }
        return null;
      }
    });
  }

  /**
   * Returns the toolkit that created this object peer.
   */
  public final SWTToolkit getToolkit() {
    return this.toolkit;
  }

  /**
   * Returns the AWT object matching this object peer.
   */
  protected Object getAWTObject() {
    return this.awtObject;
  }

  /**
   * Returns the SWT widget matching this object peer.
   */
  protected Widget getSWTWidget() {
    return this.swtWidget;
  }  

  /**
   * Returns the SWT composite associated with the parent of the AWT component of this peer.
   */
  protected abstract Widget getSWTParent();
  
  /**
   * Returns the SWT widget matching the AWT object of this peer.
   * This method is executed in SWT display event loop thread. 
   * As it's called from the constructor of this class, 
   * implementation shouldn't call any other methods than <code>getAWTObject</code>,
   * <code>getToolkit</code> or <code>getSWTParent</code>.
   */
  protected abstract Widget createSWTWidget();
  
  /**
   * Handles SWT show events.
   * This method is executed in SWT display event loop thread.  
   */
  protected void handleSWTShowEvent(Event ev) {
  }
  
  /**
   * Handles SWT hide events.
   * This method is executed in SWT display event loop thread.  
   */
  protected void handleSWTHideEvent(Event ev) {
  }
  
  /**
   * Handles SWT selection events.
   * This method is executed in SWT display event loop thread.  
   */
  protected void handleSWTSelectionEvent(Event ev) {
  }

  /**
   * Handles SWT default selection events.
   * This method is executed in SWT display event loop thread.  
   */
  protected void handleSWTDefaultSelectionEvent(Event ev) {
  }
  
  /**
   * Handles SWT menu detect events.
   * This method is executed in SWT display event loop thread.  
   */
  protected void handleSWTMenuDetectEvent(Event ev) {
  }
  
  /**
   * Returns the modifiers mask of AWT events matching SWT <code>stateMask</code>.
   */
  protected int getModifiers(int stateMask) {
    int modifiers = 0;
    if ((stateMask & SWT.COMMAND) == SWT.COMMAND) {
      modifiers |= InputEvent.META_DOWN_MASK;
    }
    if ((stateMask & SWT.CONTROL) == SWT.CONTROL) {
      modifiers |= InputEvent.CTRL_DOWN_MASK;
    }
    if ((stateMask & SWT.SHIFT) == SWT.SHIFT) {
      modifiers |= InputEvent.SHIFT_DOWN_MASK;
    }
    if ((stateMask & SWT.BUTTON1) == SWT.BUTTON1) {
      modifiers |= InputEvent.BUTTON1_DOWN_MASK;
    }
    if ((stateMask & SWT.BUTTON2) == SWT.BUTTON2) {
      modifiers |= InputEvent.BUTTON2_DOWN_MASK;
    }
    if ((stateMask & SWT.BUTTON3) == SWT.BUTTON3) {
      modifiers |= InputEvent.BUTTON3_DOWN_MASK;
    }
    return modifiers;
  }

  /**
   * Returns the AWT key code matching the given <code>swtKeyCode</code>.
   */
  protected int getAWTKeyCode(int swtKeyCode) {
    for (int [] keyCode : SWT_AWT_KEY_CODES) {
      if (keyCode [0] == swtKeyCode) {
        return keyCode [1];
      }
    }
    return KeyEvent.VK_UNDEFINED;
  }
  
  /**
   * Returns the SWT key code matching the given <code>awtKeyCode</code>.
   */
  protected int getSWTKeyCode(int awtKeyCode) {
    for (int [] keyCode : SWT_AWT_KEY_CODES) {
      if (keyCode [1] == awtKeyCode) {
        return keyCode [0];
      }
    }
    return SWT.NONE;
  }
  
  /**
   * Invokes the <code>call</code> method of <code>callable</code> in 
   * SWT display event loop thread and returns the value returned by <code>call</code>.
   */
  protected <V> V invokeInSWTDisplayThread(Callable<V> callable) {
    return this.toolkit.invokeInSWTDisplayThread(callable);
  }
  
  /**
   * Disposes the SWT widget associated with this peer. 
   */
  public void dispose() {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        if (swtWidget != null) {
          swtWidget.dispose();
          swtWidget = null;
        }
        return null;
      }
    });
  }

  /**
   * Post a given event to AWT Event queue.
   */
  protected void postAWTEvent(AWTEvent ev) {
    this.toolkit.getSystemEventQueueImpl().postEvent(ev);
  }

  /**
   * Returns an SWT image built from an AWT image.
   */
  protected org.eclipse.swt.graphics.Image getSWTImage(Image awtImage) {
    if (awtImage != null) {
      ImageObserver dummyObserver = new ImageObserver() {
        public boolean imageUpdate(Image img, int infoflags, int x,
                                   int y, int width, int height) {
          return false;
        }
      };
      final int width = awtImage.getWidth(dummyObserver);
      final int height = awtImage.getHeight(dummyObserver);
      final int [] pixels = new int [width * height];
      try {
        if (new PixelGrabber(awtImage, 0, 0, width, height, pixels, 0, width).grabPixels()) {      
          return invokeInSWTDisplayThread(new Callable<org.eclipse.swt.graphics.Image>() {
            public org.eclipse.swt.graphics.Image call() {
              byte [] data = new byte [pixels.length * 3];
              byte [] alphas = new byte [pixels.length];
              for (int i = 0, j = 0; i < pixels.length; i++) {
                alphas [i] = (byte)((pixels [i] >> 24) & 0xFF);
                data [j++] = (byte)((pixels [i] >> 16) & 0xFF); // Red
                data [j++] = (byte)((pixels [i] >> 8) & 0xFF);  // Green
                data [j++] = (byte)((pixels [i] >> 0) & 0xFF);  // Blue
              }
              PaletteData palette = new PaletteData(0xFF0000, 0x00FF00, 0x0000FF);
              ImageData imageData = new ImageData(width, height, 24, palette, width, data);
              imageData.alphaData = alphas;
              return new org.eclipse.swt.graphics.Image(getToolkit().getSWTDisplay(), imageData);
            }
          });
        }
      } catch (InterruptedException ex) {
        // Forget image
      }
    }
    return null;
  }
}
