/*
 * SWTScrollPanePeer.java 
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Adjustable;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.ScrollPane;
import java.awt.peer.ScrollPanePeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.ScrollBar;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTScrollPanePeer extends SWTContainerPeer implements ScrollPanePeer {
  public SWTScrollPanePeer(SWTToolkit toolkit, ScrollPane awtScrollPane) {
    super(toolkit, awtScrollPane);
    setScrollBarValues(awtScrollPane.getHAdjustable());
    setScrollBarValues(awtScrollPane.getVAdjustable());
  }

  @Override
  protected Widget createSWTWidget() {
    ScrollPane awtScrollPane = getAWTObject();
    ScrolledComposite composite = new ScrolledComposite(getSWTParent(), 
        awtScrollPane.getScrollbarDisplayPolicy() == ScrollPane.SCROLLBARS_NEVER 
          ? SWT.BORDER
          : SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
    // Register in toolkit scroll bars to receive SWT selection events
    getToolkit().registerSWTWidget(composite.getHorizontalBar(), this);
    getToolkit().registerSWTWidget(composite.getVerticalBar(), this);

    composite.setLayout(null);
    return composite;
  }

  @Override
  protected ScrollPane getAWTObject() {
    return (ScrollPane)super.getAWTObject();
  }
  
  @Override
  protected ScrolledComposite getSWTWidget() {
    return (ScrolledComposite)super.getSWTWidget();
  }
  
  @Override
  protected void handleSWTSelectionEvent(Event ev) {
    final Adjustable awtScrollBar;
    final int value;
    if (ev.widget == getSWTWidget().getHorizontalBar()) {
      awtScrollBar = getAWTObject().getHAdjustable();
      value = getSWTWidget().getHorizontalBar().getSelection();
    } else {
      awtScrollBar = getAWTObject().getVAdjustable();
      value = getSWTWidget().getVerticalBar().getSelection();
    }
    EventQueue.invokeLater(new Runnable() {
      public void run() {
        // Changing scroll bar value will invoke AdjustmentListeners added to scroll bar 
        awtScrollBar.setValue(value);        
      }
    });
  }
  
  public Dimension getPreferredSize() {
    return invokeInSWTDisplayThread(new Callable<Dimension>() {
      public Dimension call() {
        org.eclipse.swt.graphics.Point size = getSWTWidget().computeSize(100, 100);
        return new Dimension(size.x, size.y);
      }
    });
  }

  public int getHScrollbarHeight() {
    return invokeInSWTDisplayThread(new Callable<Integer>() {
      public Integer call() {
        return getSWTWidget().getHorizontalBar().getSize().y;
      }
    });
  }

  public int getVScrollbarWidth() {
    return invokeInSWTDisplayThread(new Callable<Integer>() {
      public Integer call() {
        return getSWTWidget().getVerticalBar().getSize().x;
      }
    });
  }

  public void childResized(final int w, final int h) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().layout();
        return null;
      }
    });
  }

  public void setScrollPosition(int x, int y) {
    // Never called by AWT
  }

  public void setValue(final Adjustable adj, final int v) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        ScrollBar scrollBar;
        if (getAWTObject().getVAdjustable() == adj) {
          scrollBar = getSWTWidget().getVerticalBar();
        } else {
          scrollBar = getSWTWidget().getHorizontalBar();
        }
        scrollBar.setSelection(v);
        return null;
      }
    });
  }
  
  public void setUnitIncrement(final Adjustable adj, final int u) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        ScrollBar scrollBar;
        if (getAWTObject().getVAdjustable() == adj) {
          scrollBar = getSWTWidget().getVerticalBar();
        } else {
          scrollBar = getSWTWidget().getHorizontalBar();
        }
        scrollBar.setIncrement(u);
        return null;
      }
    });
  }

  private void setScrollBarValues(final Adjustable adj) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        ScrollBar scrollBar;
        if (getAWTObject().getVAdjustable() == adj) {
          scrollBar = getSWTWidget().getVerticalBar();
        } else {
          scrollBar = getSWTWidget().getHorizontalBar();
        }
        if (scrollBar != null) {
          scrollBar.setValues(adj.getValue(), adj.getMinimum(), adj.getMinimum(), 
              adj.getVisibleAmount(), adj.getUnitIncrement(), adj.getBlockIncrement());
        }
        return null;
      }
    });
  }
}
