/*
 * SWTScrollbarPeer.java 
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Scrollbar;
import java.awt.event.AdjustmentEvent;
import java.awt.peer.ScrollbarPeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Slider;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTScrollbarPeer extends SWTComponentPeer implements ScrollbarPeer {
  public SWTScrollbarPeer(SWTToolkit toolkit, Scrollbar awtScrollbar) {
    super(toolkit, awtScrollbar);
    
    setValues(awtScrollbar.getValue(), awtScrollbar.getVisibleAmount(), awtScrollbar.getMinimum(), awtScrollbar.getMaximum());
    setPageIncrement(awtScrollbar.getBlockIncrement());
    setLineIncrement(awtScrollbar.getUnitIncrement());
  }

  @Override
  protected Widget createSWTWidget() {
    return new Slider(getSWTParent(), 
        getAWTObject().getOrientation() == Scrollbar.HORIZONTAL ? SWT.HORIZONTAL : SWT.VERTICAL);
  }

  @Override
  protected Scrollbar getAWTObject() {
    return (Scrollbar)super.getAWTObject();
  }

  @Override
  protected Slider getSWTWidget() {
    return (Slider)super.getSWTWidget();
  }
  
  @Override
  protected void handleSWTSelectionEvent(Event ev) {
    int value = getSWTWidget().getSelection();
    postAWTEvent(new AdjustmentEvent(getAWTObject(), 
        AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED, AdjustmentEvent.TRACK, value));
  }
  
  public void setLineIncrement(final int l) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setIncrement(l);
        return null;
      }
    });
  }

  public void setPageIncrement(final int l) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setPageIncrement(l);
        return null;
      }
    });
  }

  public void setValues(final int value, final int visible, final int minimum, final int maximum) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        Slider swtSlider = getSWTWidget();
        swtSlider.setSelection(value);
        swtSlider.setThumb(visible);
        swtSlider.setMinimum(minimum);
        swtSlider.setMaximum(maximum);
        return null;
      }
    });
  }
}
