/*
 * SWTTextField.java
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.TextArea;
import java.awt.peer.TextAreaPeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTTextAreaPeer extends SWTTextComponentPeer implements TextAreaPeer {
  public SWTTextAreaPeer(SWTToolkit toolkit, TextArea awtTextArea) {
    super(toolkit, awtTextArea);
  }

  @Override
  protected Widget createSWTWidget() {
    return new Text(getSWTParent(), SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
  }
  
  @Override
  public boolean handlesWheelScrolling() {
   return true;
 }

  public Dimension getPreferredSize(int rows, int columns) {
    FontMetrics fontMetrics = getFontMetrics(getAWTObject().getFont());
    return new Dimension(fontMetrics.charWidth('0') * columns + 20, fontMetrics.getHeight() * rows + 26);  
  }

  public Dimension preferredSize(int rows, int cols) {
    return getPreferredSize(rows, cols);
  }

  public Dimension getMinimumSize(int rows, int columns) {
    return getPreferredSize(rows, columns);
  }

  public Dimension minimumSize(int rows, int cols) {
    return getMinimumSize(rows, cols);
  }

  public void insert(final String text, final int pos) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setSelection(pos, pos);
        getSWTWidget().insert(text);
        return null;
      }
    });
  }

  public void insertText(String text, int pos) {
    insert(text, pos);
  }

  public void replaceRange(final String text, final int start, final int end) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setSelection(start, end);
        getSWTWidget().insert(text);
        return null;
      }
    });
  }

  public void replaceText(String text, int start, int end) {
    replaceRange(text, start, end);
  }
}
