/*
 * SWTTextComponentPeer.java
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Rectangle;
import java.awt.TextComponent;
import java.awt.event.TextEvent;
import java.awt.im.InputMethodRequests;
import java.awt.peer.TextComponentPeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Text;

/**
 * @author Emmanuel Puybaret
 */
public abstract class SWTTextComponentPeer extends SWTComponentPeer implements TextComponentPeer {
  public SWTTextComponentPeer(SWTToolkit toolkit, TextComponent awtTextComponent) {
    super(toolkit, awtTextComponent);
    setText(awtTextComponent.getText());
    setEditable(awtTextComponent.isEditable());
    select(awtTextComponent.getSelectionStart(), awtTextComponent.getSelectionEnd());
    
    // TODO Add text listeners
  }

  @Override
  public TextComponent getAWTObject() {
    return (TextComponent)super.getAWTObject();
  }
  
  @Override
  protected Text getSWTWidget() {
    return (Text)super.getSWTWidget();
  }
  
  protected void handleSWTModifyEvent(Event ev) {
    super.handleSWTModifyEvent(ev);
    postAWTEvent(new TextEvent(getAWTObject(), TextEvent.TEXT_VALUE_CHANGED));
  }

  @Override
  public boolean isFocusable() {
    return true;
  }

  public long filterEvents(long mask) {
    return 0;
  }

  public Rectangle getCharacterBounds(int i) {
    return null;
  }

  public int getIndexAtPoint(int x, int y) {
    return -1;
  }

  public InputMethodRequests getInputMethodRequests() {
    return null;
  }

  public int getSelectionStart() {
    return invokeInSWTDisplayThread(new Callable<Integer>() {
      public Integer call() {
        return getSWTWidget().getSelection().x;
      }
    });
  }

  public int getSelectionEnd() {
    return invokeInSWTDisplayThread(new Callable<Integer>() {
      public Integer call() {
        return getSWTWidget().getSelection().y;
      }
    });
  }

  public void select(final int selStart, final int selEnd) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setSelection(selStart, selEnd);
        return null;
      }
    });
  }

  public int getCaretPosition() {
    return invokeInSWTDisplayThread(new Callable<Integer>() {
      public Integer call() {
        return getSWTWidget().getCaretPosition();
      }
    });
  }

  public void setCaretPosition(final int pos) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setSelection(pos, pos);
        return null;
      }
    });
  }

  public void setEditable(final boolean editable) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setEditable(editable);
        return null;
      }
    });
  }

  public String getText() {
    return invokeInSWTDisplayThread(new Callable<String>() {
      public String call() {
        return getSWTWidget().getText();
      }
    });
  }

  public void setText(final String text) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setText(text);
        return null;
      }
    });
  }
}
