/*
 * SWTTextField.java
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.peer.TextFieldPeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTTextFieldPeer extends SWTTextComponentPeer implements TextFieldPeer {
  public SWTTextFieldPeer(SWTToolkit toolkit, TextField awtTextField) {
    super(toolkit, awtTextField);
    setEchoChar(awtTextField.getEchoChar());
  }

  @Override
  protected Widget createSWTWidget() {
    return new Text(getSWTParent(), SWT.SINGLE | SWT.BORDER);
  }
  
  protected void handleSWTDefaultSelectionEvent(Event ev) {
    postAWTEvent(new ActionEvent(getAWTObject(), ActionEvent.ACTION_PERFORMED, 
        getAWTObject().getText(), ev.time, getModifiers(ev.stateMask)));
  }

  public Dimension getPreferredSize(int columns) {
    FontMetrics fontMetrics = getFontMetrics(getAWTObject().getFont());
    return new Dimension(fontMetrics.charWidth('0') * columns + 24, fontMetrics.getHeight() + 14);
  }
  
  public Dimension preferredSize(int columns) {
    return getPreferredSize(columns);
  }

  public Dimension getMinimumSize(int columns) {
    return getPreferredSize(columns);
  }

  public Dimension minimumSize(int columns) {
    return getMinimumSize(columns);
  }

  public void setEchoChar(final char echoChar) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setEchoChar(echoChar);
        return null;
      }
    });
  }

  public void setEchoCharacter(final char c) {
    setEchoChar(c);
  }
}
