/*
 * SWTTrayIconPeer.java 
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Component;
import java.awt.Frame;
import java.awt.PopupMenu;
import java.awt.TrayIcon;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.peer.TrayIconPeer;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tray;
import org.eclipse.swt.widgets.TrayItem;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTTrayIconPeer extends SWTObjectPeer implements TrayIconPeer {
  private Frame trayIconMenuFrame;
  
  public SWTTrayIconPeer(SWTToolkit toolkit, TrayIcon awtTrayIcon) {
    super(toolkit, awtTrayIcon); 
    initSWTWidget();
    updateImage();
  }

  @Override
  protected Widget createSWTWidget() {
    TrayItem trayItem = new TrayItem(getSWTParent(), SWT.NONE);
    trayItem.setVisible(true);
    return trayItem;
  }

  @Override
  protected TrayIcon getAWTObject() {
    return (TrayIcon)super.getAWTObject();
  }
  
  @Override
  protected TrayItem getSWTWidget() {
    return (TrayItem)super.getSWTWidget();
  }
  
  @Override
  protected Tray getSWTParent() {
    return getToolkit().getSWTDisplay().getSystemTray();
  }

  @Override
  protected void handleSWTSelectionEvent(Event ev) {
    postAWTEvent(new ActionEvent(getAWTObject(), ActionEvent.ACTION_PERFORMED, 
        getAWTObject().getActionCommand(), ev.time, getModifiers(ev.stateMask)));
    // On Mac OS X display also menu
    if (getToolkit().isMacOSX()) {
      handleSWTMenuDetectEvent(ev);
    }
  }
  
  @Override
  protected void handleSWTMenuDetectEvent(Event ev) {
    // Create MouseEvent with a dummy component as source
    MouseEvent mouseEvent = new MouseEvent(new Component() {}, MouseEvent.MOUSE_PRESSED, ev.time, 
        getModifiers(ev.stateMask), ev.x, ev.y, 1, true, 0);
    // Retarget source to the tray icon
    mouseEvent.setSource(getAWTObject());
    postAWTEvent(mouseEvent);        
  }

  public void displayMessage(String caption, String text, String messageType) {
    // TODO Not Available in SWT ?
  }

  public void setToolTip(final String tooltip) {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().setToolTipText(tooltip);
        return null;
      }
    });
  }

  public void showPopupMenu(final int x, final int y) {
    // We can't call SWTPopupMenuPeer show method because PopupMenu getParent method always return null
    // when it's a Tray popup menu. Thus let's create a dummy frame for tray icon menu and create
    // popup menu peer if needed
    if (this.trayIconMenuFrame == null) {
      this.trayIconMenuFrame = new Frame();
      this.trayIconMenuFrame.setUndecorated(true);
      // Just create Frame peer and don't display it
      this.trayIconMenuFrame.addNotify();
    }
    invokeInSWTDisplayThread(new Callable<Void>() {      
      @SuppressWarnings("deprecation")
      public Void call() {
        PopupMenu popupMenu = getAWTObject().getPopupMenu();        
        SWTPopupMenuPeer popupMenuPeer = (SWTPopupMenuPeer)popupMenu.getPeer();
        if (popupMenuPeer == null) {
          popupMenu.addNotify();
          popupMenuPeer = (SWTPopupMenuPeer)popupMenu.getPeer();
        }
        Shell parentShell = ((SWTFramePeer)trayIconMenuFrame.getPeer()).getSWTWidget();
        popupMenuPeer.setSWTParent(parentShell);
        popupMenuPeer.initSWTWidget();

        // Display popup
        Display swtDisplay = getToolkit().getSWTDisplay();
        popupMenuPeer.getSWTWidget().setLocation(swtDisplay.getCursorLocation().x + x, 26 + y);
        popupMenuPeer.getSWTWidget().setVisible(true);

        // Let dummyFrame and popuMenu peers be disposed when SWT display is disposed 
        return null;
      }
    });
  }
  
  public void updateImage() {
    final org.eclipse.swt.graphics.Image swtImage = getSWTImage(getAWTObject().getImage());
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        org.eclipse.swt.graphics.Image oldImage = getSWTWidget().getImage();
        getSWTWidget().setImage(swtImage);
        if (oldImage != null) {
          oldImage.dispose();
        }
        return null;
      }
    });
  }
}
