/*
 * SWTWindowPeer.java 
 *
 * Copyright (c) 2007 Emmanuel PUYBARET / eTeks <info@eteks.com>. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.eteks.swtawt;

import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Window;
import java.awt.event.WindowEvent;
import java.awt.peer.WindowPeer;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Widget;

/**
 * @author Emmanuel Puybaret
 */
public class SWTWindowPeer extends SWTContainerPeer implements WindowPeer {
  public SWTWindowPeer(SWTToolkit toolkit, Window awtWindow) {
    super(toolkit, awtWindow);    
  }

  @Override
  protected Widget createSWTWidget() {
    final Window awtWidow = getAWTObject();
    int style = SWT.NO_TRIM;
    if (awtWidow.isAlwaysOnTop()) {
      style |= SWT.ON_TOP;
    }
    Shell shell = new Shell((Shell)getSWTParent(), style);
    shell.setLayout(null);
    return shell;
  }
  
  @Override
  public Window getAWTObject() {
    return (Window)super.getAWTObject();
  }
  
  @Override
  protected Shell getSWTWidget() {
    return (Shell)super.getSWTWidget();
  }
  
  /**
   * Handles SWT activate events.
   * This method is executed in SWT display event loop thread.  
   */
  @Override
  protected void handleSWTActivateEvent(Event ev) {
    super.handleSWTActivateEvent(ev);
    postAWTEvent(new WindowEvent(getAWTObject(), WindowEvent.WINDOW_ACTIVATED));
  }
  
  /**
   * Handles SWT deactivate events.
   * This method is executed in SWT display event loop thread.  
   */
  @Override
  protected void handleSWTDeactivateEvent(Event ev) {
    super.handleSWTDeactivateEvent(ev);
    postAWTEvent(new WindowEvent(getAWTObject(), WindowEvent.WINDOW_DEACTIVATED));
  }
  
  /**
   * Handles SWT close events.
   * This method is executed in SWT display event loop thread.  
   */
  protected void handleSWTCloseEvent(Event ev) {
    postAWTEvent(new WindowEvent(getAWTObject(), WindowEvent.WINDOW_CLOSED));
  }
  
  /**
   * Handles SWT iconify events.
   * This method is executed in SWT display event loop thread.  
   */
  protected void handleSWTIconifyEvent(Event ev) {
    postAWTEvent(new WindowEvent(getAWTObject(), WindowEvent.WINDOW_ICONIFIED));        
  }
  
  /**
   * Handles SWT deiconify events.
   * This method is executed in SWT display event loop thread.  
   */
  protected void handleSWTDeiconifyEvent(Event ev) {
    postAWTEvent(new WindowEvent(getAWTObject(), WindowEvent.WINDOW_DEICONIFIED));        
  }

  @Override
  public boolean isFocusable() {
    return true;
  }

  public boolean requestWindowFocus() {
    return invokeInSWTDisplayThread(new Callable<Boolean>() {
      public Boolean call() {
        return getSWTWidget().setFocus();
      }
    });
  }

  public void updateFocusableWindowState() {
    // TODO Manage focusable state
  }

  public void setModalBlocked(Dialog blocker, boolean blocked) {
    // TODO Manage modal blocked

  }

  public void setAlwaysOnTop(boolean alwaysOnTop) {
    // TODO No way to change ON_TOP style ?
  }

  public void updateMinimumSize() {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        Dimension minimumSize = getAWTObject().getMinimumSize();
        getSWTWidget().setMinimumSize(minimumSize.width, minimumSize.height);
        return null;
      }
    });
  }
  
  public void toBack() {
    // TODO Is there a way to achieve that in SWT ?
  }

  public void toFront() {
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        getSWTWidget().forceActive();
        return null;
      }
    });
  }

  public void updateIconImages() {
    List<Image> awtImages = getAWTObject().getIconImages();
    final List<org.eclipse.swt.graphics.Image> swtImages = new ArrayList<org.eclipse.swt.graphics.Image>(awtImages.size());
    for (int i = 0; i < awtImages.size(); i++) {
      org.eclipse.swt.graphics.Image swtImage = getSWTImage(awtImages.get(i));
      if (swtImage != null) {
        swtImages.add(swtImage);
      }
    }
    invokeInSWTDisplayThread(new Callable<Void>() {
      public Void call() {
        org.eclipse.swt.graphics.Image [] oldImages = getSWTWidget().getImages();
        getSWTWidget().setImages(swtImages.toArray(new org.eclipse.swt.graphics.Image [swtImages.size()]));
        for (org.eclipse.swt.graphics.Image oldImage : oldImages) {
          oldImage.dispose();
        }
        return null;
      }
    });
  }
}
