/**
 * SWTToolkitTest.java
 */
package com.eteks.test;

import java.awt.AWTException;
import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Canvas;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.CheckboxMenuItem;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Desktop;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.FileDialog;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Label;
import java.awt.List;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.Panel;
import java.awt.PopupMenu;
import java.awt.RenderingHints;
import java.awt.ScrollPane;
import java.awt.Scrollbar;
import java.awt.SystemTray;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.Toolkit;
import java.awt.TrayIcon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.net.URI;
import java.util.ArrayList;
import java.util.Arrays;

/**
 * A small test that tests most AWT features available in Java SE 6.
 * @author Emmanuel Puybaret
 */
public class SWTToolkitTest {
  public static void main(String [] args) throws AWTException {
    Toolkit.getDefaultToolkit().beep();
    Toolkit.getDefaultToolkit().setDynamicLayout(true);
    System.out.println(Toolkit.getDefaultToolkit().getScreenResolution());
    System.out.println(Toolkit.getDefaultToolkit().getScreenSize());
    System.out.println(Toolkit.getDefaultToolkit().getScreenInsets(GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration()));
    
    BufferedImage frameImage = new BufferedImage(16, 16, BufferedImage.TYPE_INT_ARGB);
    Graphics2D graphics = (Graphics2D)frameImage.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    graphics.setStroke(new BasicStroke(2));
    graphics.drawOval(1, 1, 13, 13);
    graphics.dispose();
    
    BufferedImage systemTrayImage = new BufferedImage(16, 16, BufferedImage.TYPE_INT_ARGB);
    graphics = (Graphics2D)systemTrayImage.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    graphics.setColor(Color.BLACK);
    graphics.setStroke(new BasicStroke(1));
    graphics.fillOval(2, 2, 11, 11);
    graphics.dispose();
    
    final Frame frame = new Frame("Test");
    try {
      frame.setIconImages(Arrays.asList(new Image[] {frameImage}));
      // Create a popup menu for Tray
      PopupMenu popup = new PopupMenu();
      MenuItem trayMenuItem = new MenuItem("Tray Menu item");
      trayMenuItem.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          System.out.println(e);
        }
      });
      popup.add(trayMenuItem);
      // Construct a TrayIcon
      TrayIcon trayIcon = new TrayIcon(systemTrayImage, "Tray Demo", popup);
      SystemTray.getSystemTray().add(trayIcon);
    } catch (LinkageError ex) {
      // Just in case that these Java SE 6 features are run under a Java SE 5 JVM
      ex.printStackTrace();
    }
    
    Button drawItButton = new Button("Draw it !");
    Button fileDialogButton = new Button("File dialog");
    final TextArea textArea = new TextArea("Bla bla", 3, 20);
    textArea.setBackground(Color.YELLOW);
    textArea.setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
    final List list = new List(5, true);
    for (String font : Toolkit.getDefaultToolkit().getFontList()) {
      list.add(font);
    }
    final Choice choice = new Choice();
    choice.add("View eteks.com");
    choice.add("Write to puybaret@eteks.com");
    choice.add("Open image.png");
    Checkbox checkbox = new Checkbox("Check box");
    CheckboxGroup group = new CheckboxGroup();
    TextField textField = new TextField("Bla bla");
    Scrollbar scrollbar = new Scrollbar(Scrollbar.HORIZONTAL);
    
    // Create a menu bar
    MenuBar menuBar = new MenuBar();
    Menu menu1 = new Menu("Menu 1");
    Menu menu2 = new Menu("Menu 2");
    Menu menu3 = new Menu("Sub menu");
    menu1.add("Menu item 1");
    menu1.addSeparator();
    menu1.add("Menu item 2");
    CheckboxMenuItem checkboxMenuItem = new CheckboxMenuItem("Check box");
    checkboxMenuItem.setShortcut(new MenuShortcut(KeyEvent.VK_F2, true));
    menu2.add(checkboxMenuItem);
    menu2.add(menu3);
    menu3.add(new MenuItem("Sub menu item 1"));
    menu3.add(new MenuItem("Sub menu item 2"));
    menuBar.add(menu1);
    menuBar.add(menu2);
    frame.setMenuBar(menuBar);
    
    // Create a popup menu
    final PopupMenu popupMenu = new PopupMenu();
    popupMenu.add("Menu item 1");
    Menu popupMenu2 = new Menu("Sub menu");
    popupMenu2.add("Menu item 1");
    popupMenu2.addSeparator();
    popupMenu2.add("Menu item 2");
    popupMenu.add(popupMenu2);

    Panel topPanel = new Panel();
    topPanel.add(new Label("Label"));   
    topPanel.add(checkbox);
    topPanel.add(new Checkbox("Radio 1", group, true));
    topPanel.add(new Checkbox("Radio 2", group, true));
    topPanel.add(choice);
    topPanel.add(scrollbar);
    
    Panel westPanel = new Panel();
    westPanel.setLayout(new GridLayout(2, 1));
    westPanel.add(drawItButton);
    westPanel.add(fileDialogButton);
    
    frame.add(topPanel, BorderLayout.NORTH);
    frame.add(textField, BorderLayout.SOUTH);
    frame.add(textArea, BorderLayout.CENTER);
    frame.add(westPanel, BorderLayout.WEST);
    frame.add(list, BorderLayout.EAST);
    frame.pack();
    frame.setVisible(true);

    // Add some listeners to track events
    drawItButton.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        System.out.println(e);
      }
      public void mousePressed(MouseEvent e) {
        System.out.println(e);
      }
      public void mouseReleased(MouseEvent e) {
        System.out.println(e);
      }
    });
    drawItButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        textArea.insert("oo", 2);
        final Dialog dialog = new Dialog(frame, "Dialog", true);
        dialog.setResizable(true);
        dialog.setBackground(Color.LIGHT_GRAY);
        dialog.setFont(new Font("Helvetica", Font.BOLD, 11));
        dialog.add(new DrawIt());
        dialog.pack();
        dialog.addWindowListener(new WindowAdapter() {
          public void windowClosing(WindowEvent e) {
            dialog.dispose();
          }
        });
        dialog.setVisible(true);
      }
    });
    fileDialogButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        FileDialog dialog = new FileDialog(frame, "Save", FileDialog.SAVE);
        dialog.setFile(new File("test").toString());
        dialog.setVisible(true);
        System.out.println(dialog.getFile());
      }
    });
    
    textArea.addTextListener(new TextListener() {
      public void textValueChanged(TextEvent e) {
        System.out.println(textArea.getText());
      }
    });
    choice.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        try {
          if (e.getItem().equals("View eteks.com")) {
            Desktop.getDesktop().browse(
                new URI("http://www.eteks.com"));
          } else if (e.getItem().equals(
              "Write to puybaret@eteks.com")) {
            Desktop.getDesktop().mail(
                new URI("mailto:puybaret@eteks.com"));
          } else {
            Desktop.getDesktop().open(
                new File("test/com/eteks/test/image.jpg"));
          }
        } catch (Exception ex) {
          ex.printStackTrace();
        }
      }
    });
    list.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        System.out.println(e + " " + Arrays.deepToString(list.getSelectedItems()));
      }
    });
    ActionListener actionListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.out.println(e);
      }
    };
    checkboxMenuItem.addActionListener(actionListener);
    list.addActionListener(actionListener);
    textField.addActionListener(actionListener);
    checkboxMenuItem.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        System.out.println(e);
      }
    });
    checkbox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        System.out.println(e);
      }
    });
    frame.add(popupMenu);
    list.addMouseListener(new MouseAdapter() {
      @Override
      public void mousePressed(MouseEvent e) {
        if (e.isPopupTrigger()) {
          popupMenu.show((Component) e.getSource(), e.getX(), e.getY());
        }
      }

      @Override
      public void mouseReleased(MouseEvent e) {
        mousePressed(e);
      }
    });
    scrollbar.addAdjustmentListener(new AdjustmentListener() {
      public void adjustmentValueChanged(AdjustmentEvent e) {
        System.out.println(e);
      }
    });
  }  

  private static class DrawIt extends Panel {
    public DrawIt() {
      setLayout(new BorderLayout());
      CheckboxGroup drawGroup  = new CheckboxGroup();

      ScrollPane scrollPane = new ScrollPane();
      final Canvas drawItcomponent = new DrawItComponent(drawGroup);
      scrollPane.add(drawItcomponent);
      add(scrollPane);

      java.util.List<Checkbox> checkboxes = new ArrayList<Checkbox>();
      checkboxes.add(new Checkbox("Line", drawGroup, true));
      checkboxes.add(new Checkbox("Rectangle", drawGroup, false));
      checkboxes.add(new Checkbox("Rectangle 3D", drawGroup, false));
      checkboxes.add(new Checkbox("Polygon", drawGroup, false));
      checkboxes.add(new Checkbox("Oval", drawGroup, false));
      checkboxes.add(new Checkbox("Arc", drawGroup, false));
      checkboxes.add(new Checkbox("Text", drawGroup, false));
      
      Panel panel = new Panel();
      panel.setLayout(new GridLayout(7, 1));
      for (Checkbox checkbox : checkboxes) {
        panel.add(checkbox);
        checkbox.addItemListener(new ItemListener() {
          public void itemStateChanged(ItemEvent e) {
            drawItcomponent.repaint();
          }
        });
      }
      add(panel, BorderLayout.EAST);
    }
  }

  private static class DrawItComponent extends Canvas {
    private CheckboxGroup drawGroup;
    private int           xStart, yStart, xEnd, yEnd;
    private Color         drawColor;
    private int           angleStart, angle;

    DrawItComponent(CheckboxGroup choixDessin) {
      this.drawGroup = choixDessin;
      setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
    }

    public boolean mouseDown(Event evt, int x, int y) {
      this.xStart = this.xEnd = x;
      this.yStart = this.yEnd = y;
      this.drawColor = new Color((float)Math.random(), (float)Math.random(), (float)Math.random());
      this.angleStart = (int)(Math.random() * 360);
      this.angle = (int)(Math.random() * 360);
      repaint();
      return true;
    }

    public boolean mouseDrag(Event evt, int x, int y) {
      this.xEnd = x;
      this.yEnd = y;
      repaint();
      return true;
    }

    public void paint(Graphics gc) {
      String drawType = drawGroup.getSelectedCheckbox().getLabel();
      gc.setColor(drawColor);
      int x = Math.min(this.xStart, this.xEnd);
      int y = Math.min(this.yStart, this.yEnd);
      int width = Math.abs(this.xEnd - this.xStart);
      int height = Math.abs(this.yEnd - this.yStart);
      if (drawType.equals("Line")) {
        gc.drawLine(this.xStart, this.yStart, this.xEnd, this.yEnd);
      } else if (drawType.equals("Rectangle")) {
        gc.fillRect(x, y, width, height);
      } else if (drawType.equals("Rectangle 3D")) {
        gc.fill3DRect(x, y, width, height, true);
      } else if (drawType.equals("Polygon")) {
        int xPoints[] = {x, x, x + width, x + width};
        int yPoints[] = {y, y + height, y, y + height};
        gc.fillPolygon(xPoints, yPoints, xPoints.length);
      } else if (drawType.equals("Oval")) {
        gc.fillOval(x, y, width, height);
      } else if (drawType.equals("Arc")) {
        gc.drawOval(x, y, width, height);
        gc.fillArc(x, y, width, height, angleStart, angle);
      } else if (drawType.equals("Text")) {
        gc.setFont(new Font("Lucinda Sans", Font.PLAIN, height));
        FontMetrics tailles = gc.getFontMetrics();
        gc.drawString("Text", x, y + tailles.getAscent() + tailles.getLeading());
      }
    }

    @Override
    public Dimension getPreferredSize() {
      return new Dimension(500, 500);
    }
  }
}
